#include "image3d.h"
#include "imgfiles.h"
#include <stdio.h>
#include <iostream>
#include <iomanip>

using namespace std;
using namespace i3d;

void usage(const char *str)
{
   cerr << str << " ... simple image checker \n" 
	"Copyright (C) 2023 Centre for Biomedical Image Analysis \n"
	"This is free software; see the source for copying conditions. "
	"There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR "
	"A PARTICULAR PURPOSE.\n\n";
	 cerr << "Usage:" << endl << endl;
	 cerr << "$> " << str << " <input file> [simplify]" << endl << endl;
	 cerr << "Options:" << endl << endl;
	 cerr << "simplify ... reduces the amount of printed information" << endl << endl;
	 cerr << endl;
}

void showinfo(bool simple, const char *dtype, size_t dynamic_range, size_t dimensions)
{
	if (simple)
	{
		cout << dtype << " " << dynamic_range << " " << dimensions << endl;
	}
	else
	{
		cout << "+------------+-------------------+-----------+" << endl;
		cout << "|data type   |dynamic range      |dimensions |" << endl;
		cout << "+------------+-------------------+-----------+" << endl;
		cout << "|" << std::left << setw(12) << dtype << "|" << std::right << 
				setw(19) << dynamic_range << "|" << setw(11) << dimensions << "|" << endl;
		cout << "+------------+-------------------+-----------+" << endl;
	}		
}

int main (int argc, char *argv[])
{
	 if (argc < 2)
	 {
		  usage(argv[0]);
		  exit(-1);
	 }
  try
  {
    ImageHeader ih = ReadImageHeader(argv[1]);
	 int dim;

    if (ih.size.z == 1)
		dim = 2;
	 else
      dim = 3;

	 bool simplify = false;
	 if ((argc == 3) && (strcmp(argv[2], "simplify") == 0))
	 {
		simplify = true;
	 }

	 switch (ih.type)
      {
	  case BinaryVoxel:
				 {
					   Image3d<bool> img(argv[1]);
						showinfo(simplify, "binary", img.GetDynamicRange(), dim);
					   break;
				 }
	  case Gray8Voxel:
				 {
					   Image3d<GRAY8> img(argv[1]);
						showinfo(simplify, "grayscale", img.GetDynamicRange(), dim);
					   break;
				 }
	  case Gray16Voxel:
				 {
					   Image3d<GRAY16> img(argv[1]);
						showinfo(simplify, "grayscale", img.GetDynamicRange(), dim);
					   break;
				 }
	  case RGBVoxel:
				 {
					   Image3d<RGB> img(argv[1]);
						showinfo(simplify, "rgb", img.GetDynamicRange(), dim);
					   break;
				 }
	  case RGB16Voxel:
				 {
					   Image3d<RGB16> img(argv[1]);
						showinfo(simplify, "rgb", img.GetDynamicRange(), dim);
					   break;
				 }
	  case FloatVoxel:
				 {
					   Image3d<float> img(argv[1]);
						showinfo(simplify, "float", img.GetDynamicRange(), dim);
					   break;
				 }
      default:
				 {
					  cerr << argv[0] << ": Unsupported data type." << endl;
					  exit(-1);
				 }
      }
  }
  catch (IOException & e)
  {
    cerr << e << endl;
    exit (-1);
  }
  catch (InternalException & e)
  {
    cerr << e << endl;
    exit (-1);
  }
  catch (bad_alloc &)
  {
    cerr << "Not enough memory." << endl;
    exit (-1);
  }
  catch (...)
  {
    cerr << "Unknown exception." << endl;
    exit (-1);
  }

  return 0;
}
