#include "image3d.h"
#include "imgfiles.h"
#include <stdio.h>
#include <iostream>

using namespace std;
using namespace i3d;

void usage(const char *str)
{
   cerr << str << " ... simple image conversion tool \n" 
	"Copyright (C) 2010 Centre for Biomedical Image Analysis \n"
	"This is free software; see the source for copying conditions. "
	"There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR "
	"A PARTICULAR PURPOSE.\n\n";
	 cerr << "Usage:" << endl << endl;
	 cerr << "$> " << str << " <input file> <output file> [version] " 
				<< endl << endl;
	 cerr << " - parameter 'version' is available for ICS format only" << endl;
	 cerr << "   and its possible values are '1' or '2'. The latter one" << endl;
	 cerr << "   is the implicit option." << endl;
	 cerr << endl;
	 cerr << "EXAMPLE 1:" << endl;
	 cerr << "$> " << str << " obrazek.tif obrazek.ics 2 " << endl;
	 cerr << endl;
	 cerr << "EXAMPLE 2:" << endl;
	 cerr << "$> " << str << " obrazek.jpg obrazek.tif" << endl;
	 cerr << endl;
}

int main (int argc, char *argv[])
{
	 if (argc < 3)
	 {
		  usage(argv[0]);
		  exit(-1);
	 }
  try
  {
	 cerr << "Checking voxel type" << endl;

    ImgVoxelType vt = ReadImageType (argv[1]);
    cerr << "Voxel=" << VoxelTypeToString (vt) << endl;

	 FileFormat ff = GuessFileFormat(argv[2]);

	 if (argc == 4)
	 {
		 if ((ff == IMG_ICSv1) || (ff == IMG_ICSv2))
		 {
			  switch (atoi(argv[3]))
			  {
					case 1: ff = IMG_ICSv1;
							  break;
					case 2: ff = IMG_ICSv2;
							  break;
					default: cerr << argv[0] << ": unsupported version." << endl;
								exit(-1);
			  }
		 }
		 else
		 {
			  cerr << "warning: parameter version ignored" << endl;
		 }
	 }

    switch (vt)
      {
	  case BinaryVoxel:
				 {
					  Image3d<bool> img(argv[1]);
					  img.SaveImage(argv[2], ff);
					  break;
				 }
	  case Gray8Voxel:
				 {
							Image3d <GRAY8> img (argv[1]);
							img.SaveImage (argv[2], ff);
							break;
				 }
	  case Gray16Voxel:
				 {
							Image3d <GRAY16> img (argv[1]);
							img.SaveImage (argv[2], ff);
							break;
				 }
	  case RGBVoxel:
				 {
							Image3d <RGB> img (argv[1]);
							img.SaveImage (argv[2], ff);
							break;
				 }
	  case RGB16Voxel:
				 {
							Image3d <RGB16> img (argv[1]);
							img.SaveImage (argv[2], ff);
							break;
				 }
	  case FloatVoxel:
				 {
							Image3d <float > img (argv[1]);
							img.SaveImage (argv[2], ff);
							break;
				 }
      default:
				 {
					  cerr << argv[0] << ": Unsupported voxel type." << endl;
					  exit(-1);
				 }
      }
  }
  catch (IOException & e)
  {
    cerr << e << endl;
    exit (-1);
  }
  catch (InternalException & e)
  {
    cerr << e << endl;
    exit (-1);
  }
  catch (bad_alloc &)
  {
    cerr << "Not enough memory." << endl;
    exit (-1);
  }
  catch (...)
  {
    cerr << "Unknown exception." << endl;
    exit (-1);
  }

  return 0;
}
