/*
 * i3dlib - image manipulation library
 *
 * Copyright (C) 2000-2015   Centre for Biomedical Image Analysis (CBIA)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  FILE: mpeg7ehdtest.cc
 *
 */


#include "image3d.h"
#include "descriptors/mpeg7ehd.h"

using namespace std;
using namespace i3d;

int main(int argc, char* argv[])
{
	if (argc < 2 ) {
		cerr << argv[0] << " ... evaluation of MPEG-7 EHD features \n\n"
		"Copyright (C) 2015 Centre for Biomedical Image Analysis \n"
		"This is free software; see the source for copying conditions. "
		"There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR "
		"A PARTICULAR PURPOSE.\n\n";
		cerr << "Usage: "<< argv[0] << " <input image>" << endl;
		cerr << " - input image: the analyzed texture" << endl << endl;
		exit(1);
	}

	try  
	{
		ImgVoxelType vt = ReadImageType(argv[1]);
		vector<double> final_vector;
		switch (vt)
		{
			case Gray8Voxel: 
			{
				Image3d<GRAY8> img(argv[1]);
				if (img.GetSizeZ() > 1) Compute3DEHD(img, final_vector);
				else Compute2DEHD(img, final_vector);				
				break;
			}
			case Gray16Voxel: 
			{
				Image3d<GRAY16> img(argv[1]);
				if (img.GetSizeZ() > 1) Compute3DEHD(img, final_vector);
				else Compute2DEHD(img, final_vector);
				break;
			}
			default: cerr << "wrong image type" << endl;
		}

		// print the feature vector
		for (unsigned i = 0; i < final_vector.size(); i++)
		{
			if (i > 0) cout << ",";		
			cout << final_vector[i]; 
		}
		cout << endl; 
	}
	catch (IOException& e) {
		cerr << e <<endl;
		exit(1);
	}
	catch (InternalException& e) {
		cerr << e <<endl;
		exit(1);
	}
	catch (bad_alloc&) {
		cerr << "Not enough memory." << endl;
		exit(1);
	}
	catch (...) {
		cerr << "mpeg7ehdtest: Unknown exception." << endl;
		exit(1);
	}

  return 0;
}