/*
 * i3dlib - image manipulation library
 *
 * Copyright (C) 2000-2006   Centre for Biomedical Image Analysis (CBIA)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * result_creator.cpp
 *
 * Result creator
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006 - 2011
 */

#include "result_creator.h"

namespace i3d
{
	namespace fls
	{
		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                     Class ResultCreator                     //
		//                                                             //
		/////////////////////////////////////////////////////////////////

		/**********************************************************************/

		void ResultCreator::GetMask(Type::BinVoxel *mask, ResType type, bool only_in_domain) const
		{
			if (only_in_domain)
			{
				switch (type)
				{
				case Interior: 

					InteriorMaskInDomain(mask); 
					break;

				case Interface: 

					InterfaceMaskInDomain(mask); 
					break;

				case Exterior: 

					ExteriorMaskInDomain(mask);
					break;

				case InteriorAndInterface: 

					InteriorAndInterfaceMaskInDomain(mask);
					break;

				case ExteriorAndInterface: 

					ExteriorAndInterfaceMaskInDomain(mask);
					break;

				case InteriorAndExterior: 

					InteriorAndExteriorMaskInDomain(mask);
					break;
				}
			}
			else
			{
				switch (type)
				{
				case Interior: 

					InteriorMask(mask);
					break;

				case Interface: 

					InterfaceMask(mask);
					break;

				case Exterior: 

					ExteriorMask(mask);
					break;

				case InteriorAndInterface: 

					InteriorAndInterfaceMask(mask);
					break;

				case ExteriorAndInterface: 

					ExteriorAndInterfaceMask(mask);
					break;

				case InteriorAndExterior: 

					InteriorAndExteriorMask(mask);
					break;
				}
			}
		}

		/**********************************************************************/

		void ResultCreator::GetLabels(Type::LabVoxel *labels, bool only_in_domain) const
		{
			if (only_in_domain)
			{
				Labels(labels);
			}
			else
			{
				LabelsInDomain(labels);
			}
		}

		/**********************************************************************/

	} // namespace fls 

} // namespace i3d

